@@ -128,10 +128,10 @@ gnome_print_config_get_boolean (GnomePrintConfig *config, const guchar *key, gbo
 	v = gnome_print_config_get (config, key);
 
 	if (v != NULL) {
-		if (!strcasecmp (v, "true") ||
-		    !strcasecmp (v, "yes") ||
-		    !strcasecmp (v, "y") ||
-		    !strcasecmp (v, "yes") ||
+		if (!g_ascii_strcasecmp (v, "true") ||
+		    !g_ascii_strcasecmp (v, "yes") ||
+		    !g_ascii_strcasecmp (v, "y") ||
+		    !g_ascii_strcasecmp (v, "yes") ||
 		    (atoi (v) != 0)) {
 			*val = TRUE;
 			return TRUE;
@@ -208,12 +208,12 @@ gnome_print_config_get_length (GnomePrintConfig *config, const guchar *key, gdou
 	v = gnome_print_config_get (config, key);
 
 	if (v != NULL) {
-		gchar *loc, *e;
+		guchar *loc, *e;
 		loc = setlocale (LC_NUMERIC, NULL);
 		setlocale (LC_NUMERIC, "C");
-		*val = strtod (v, &e);
+		*val = strtod (v, (gchar **) &e);
 		setlocale (LC_NUMERIC, loc);
-		if (e == (gchar *) v) {
+		if (e == v) {
 			g_free (v);
 			return FALSE;
 		}
