@@ -71,30 +71,6 @@ static GnomePrintContextClass *parent_class = NULL;
 #define PAGE_HEADER_SIZE (PAGE_SIGNATURE_SIZE + 4)
 #define BLOCKSIZE 4096
 
-#if 0
-typedef struct {
-	char signature [GNOME_METAFILE_SIGNATURE_SIZE];
-	gint32  size;
-} GnomeMetaFileHeader;
-
-/* this is used to quickly scan pages through a metafile, it is
-   stored at the start of the file, and after every showpage */
-typedef struct {
-	char signature[4];	/* magic token for a page header */
-	gint32 size;
-} GnomeMetaPageHeader;
-#endif
-
-#if 0
-/* ENCODED sizes maybe != structure sizes (alignment) */
-#define FILEHEADER_SIZE (GNOME_METAFILE_SIGNATURE_SIZE+4)
-#define PAGEHEADER_SIZE (4+4)
-#endif
-
-#if 0
-static const char *decode_header (const char *data, GnomeMetaFileHeader *mh);
-#endif
-
 typedef enum {
 	GNOME_META_BEGINPAGE,
 	GNOME_META_SHOWPAGE,
@@ -145,13 +121,14 @@ static void gpm_encode_string (GnomePrintContext *pc, const guchar *str);
 static void gpm_encode_int (GnomePrintContext *pc, gint32 value);
 static void gpm_encode_double (GnomePrintContext *pc, double d);
 static void gpm_encode_block (GnomePrintContext *pc, const guchar *data, gint size);
+#ifdef SMOKING_LOTS_OF_CRACK
 static void gpm_encode_pointer (GnomePrintContext *ctx, gpointer ptr);
+#endif
 static gboolean gpm_ensure_space (GnomePrintMeta *meta, int size);
 static void gpm_encode_bpath (GnomePrintContext *pc, const ArtBpath *bpath);
 static const char *gpm_decode_bpath (const char *data, ArtBpath **bpath);
 
 static const guchar *gpm_decode_string (const guchar *data, guchar **dest);
-static const guchar *gpm_decode_pointer (const guchar *data, gpointer *ptr);
 
 /* Encoding and decoding of header data */
 static void gpm_encode_int_header (GnomePrintContext *pc, gint32 value);
@@ -581,7 +558,6 @@ static const char *
 decode_double (const char *data, double *dest)
 {
 	int i;
-	int sd = sizeof (double);
 	data = decode_int (data, &i);
 
 	switch ((GnomeMetaDoubleType)i) {
@@ -595,15 +571,16 @@ decode_double (const char *data, double *dest)
 		break;
 	case GNOME_META_DOUBLE_I386:
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-		for (i = 0; i < sd; i++)
-			((guint8 *)dest)[sd - 1 - i] = data[i];
-		data += sd;
+		g_assert (sizeof (double) == 8);
+		for (i = 0; i < sizeof (double); i++)
+			((guint8 *)dest)[sizeof (double) - 1 - i] = data[i];
 #elif G_BYTE_ORDER == G_LITTLE_ENDIAN
-		memcpy (dest, data, sd);
-		data += sd;
+		g_assert (sizeof (double) == 8);
+		memcpy (dest, data, sizeof (double));
 #else
 #error decode_double_needs_attention
 #endif
+		data += 8; /* An i386 double is eight bytes.  */
 		break;
 	default:
 		*dest = 0;   /* ??? */
@@ -1190,30 +1167,30 @@ gpm_encode_int (GnomePrintContext *pc, gint32 value)
 static void
 gpm_encode_double (GnomePrintContext *pc, double d)
 {
-	int sd = sizeof (d);
-
 	if (d == (gint32)d) {
 		gpm_encode_int (pc, GNOME_META_DOUBLE_INT);
-		gpm_encode_int (pc, (int)d);
+		gpm_encode_int (pc, (gint32)d);
 	} else {
 		double d1000 = d * 1000;
 		if (d1000 == (gint32)d1000) {
 			gpm_encode_int (pc, GNOME_META_DOUBLE_INT1000);
-			gpm_encode_int (pc, (int)d1000);
+			gpm_encode_int (pc, (gint32)d1000);
 		} else {
 			gpm_encode_int (pc, GNOME_META_DOUBLE_I386);
 #if G_BYTE_ORDER == G_BIG_ENDIAN
+			g_assert (sizeof (double) == 8);
 			{
-				int     i;
-				guint8 *t  = (guint8 *)&d;
-				guint8 block[sizeof (d)];
+				int i;
+				const guchar *t  = (guchar *)&d;
+				guchar block[sizeof (d)];
 
-				for (i = 0; i < sd; i++)
-					block[sd - 1 - i] = t[i];
-				gpm_encode_block (pc, sd, &block);
+				for (i = 0; i < sizeof (d); i++)
+					block[sizeof (d) - 1 - i] = t[i];
+				gpm_encode_block (pc, block, sizeof (d));
 			}
 #elif G_BYTE_ORDER == G_LITTLE_ENDIAN
-			gpm_encode_block (pc, (const guchar *) &d, sd);
+			g_assert (sizeof (double) == 8);
+			gpm_encode_block (pc, (const guchar *) &d, sizeof (d));
 #else
 #error encode_double_needs_attention
 #endif
