@@ -25,7 +25,12 @@
  *
  */
 
+#define noGPA_MEDIA_DEBUG
+
 #include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <time.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include "../gnome-print-i18n.h"
@@ -40,25 +45,63 @@ static GPANode *gpa_media_init_physicalorientations (GPANode *media);
 static GPANode *gpa_media_init_logicalorientations (GPANode *media);
 static GPANode *gpa_media_init_layouts (GPANode *media);
 
+static GPANode *media = NULL;
+static time_t timestamp;
+static time_t lastcheck;
+
+static void
+gpa_media_gone (gpointer data, GObject *gone)
+{
+#ifdef GPA_MEDIA_DEBUG
+	g_print ("GPAMedia: Media %p is gone\n", gone);
+#endif
+
+	media = NULL;
+}
+
 GPANode *
 gpa_media_load (void)
 {
-	GPANode *media, *physicalsizes, *physicalorientations, *logicalorientations, *layouts;
+	GPANode *physicalsizes, *physicalorientations, *logicalorientations, *layouts;
 	xmlDocPtr doc;
 	xmlNodePtr root;
+	struct stat s;
+
+	if (media) {
+		if (time (NULL) == lastcheck) {
+			return gpa_node_ref (media);
+		} else {
+			lastcheck = time (NULL);
+			if (!stat (DATADIR "/gnome-print-2.0/media/media.xml", &s)) {
+				if (s.st_mtime == timestamp) {
+#ifdef GPA_MEDIA_DEBUG
+					g_print ("GPAMedia: Found valid media %p\n", media);
+#endif
+					return gpa_node_ref (media);
+				}
+			}
+			g_object_weak_unref (G_OBJECT (media), gpa_media_gone, &media);
+			media = NULL;
+		}
+	}
 
 	/* Initialize Media */
 	media = gpa_option_node_new ("Media");
-
+	g_object_weak_ref (G_OBJECT (media), gpa_media_gone, &media);
+	/* Bookeeping */
+	lastcheck = time (NULL);
+	if (!stat (DATADIR "/gnome-print-2.0/media/media.xml", &s)) {
+		timestamp = s.st_mtime;
+	}
+#ifdef GPA_MEDIA_DEBUG
+	g_print ("GPAMedia: Created new media %p\n", media);
+#endif
 	/* Initialize PhysicalSizes */
 	physicalsizes = gpa_media_init_physicalsizes (media);
-
 	/* Initialize PhysicalOrientations */
 	physicalorientations = gpa_media_init_physicalorientations (media);
-
 	/* Initialize LogicalOrientations */
 	logicalorientations = gpa_media_init_logicalorientations (media);
-
 	/* Initialize Layouts */
 	layouts = gpa_media_init_layouts (media);
 
