@@ -30,6 +30,7 @@
 #include <libxml/globals.h>
 #include "gpa-utils.h"
 #include "gpa-value.h"
+#include "gpa-key.h"
 #include "gpa-option.h"
 
 static void gpa_option_class_init (GPAOptionClass *klass);
@@ -43,6 +44,8 @@ static GPANode *gpa_option_get_child (GPANode *node, GPANode *ref);
 static GPANode *gpa_option_lookup (GPANode *node, const guchar *path);
 static void gpa_option_modified (GPANode *node, guint flags);
 
+static GPANode *gpa_option_create_key_private (GPAOption *option);
+
 static GPAOption *gpa_option_new_node_from_tree (xmlNodePtr tree, const guchar *id);
 static GPAOption *gpa_option_new_key_from_tree (xmlNodePtr tree, const guchar *id);
 static GPAOption *gpa_option_new_list_from_tree (xmlNodePtr tree, const guchar *id);
@@ -100,6 +103,8 @@ gpa_option_class_init (GPAOptionClass *klass)
 	node_class->get_child = gpa_option_get_child;
 	node_class->lookup = gpa_option_lookup;
 	node_class->modified = gpa_option_modified;
+
+	klass->create_key = gpa_option_create_key_private;
 }
 
 static void
@@ -343,6 +348,79 @@ gpa_option_modified (GPANode *node, guint flags)
 	}
 }
 
+static GPANode *
+gpa_option_create_key_private (GPAOption *option)
+{
+	GPAKey *key;
+	GPANode *child;
+	GSList *l;
+
+	key = NULL;
+
+	switch (option->type) {
+	case GPA_OPTION_TYPE_NODE:
+	case GPA_OPTION_TYPE_KEY:
+	case GPA_OPTION_TYPE_STRING:
+		key = (GPAKey *) gpa_node_new (GPA_TYPE_KEY, GPA_NODE_ID (option));
+		key->option = GPA_NODE (option);
+		gpa_node_ref (key->option);
+		if (option->value) key->value = g_strdup (option->value);
+		l = NULL;
+		for (child = option->children; child != NULL; child = child->next) {
+			GPANode *kch;
+			kch = gpa_option_create_key (GPA_OPTION (child));
+			if (kch) {
+				l = g_slist_prepend (l, kch);
+			}
+		}
+		while (l) {
+			GPANode *kch;
+			kch = GPA_NODE (l->data);
+			l = g_slist_remove (l, kch);
+			kch->parent = GPA_NODE (key);
+			kch->next = key->children;
+			key->children = kch;
+		}
+		break;
+	case GPA_OPTION_TYPE_LIST:
+		key = (GPAKey *) gpa_node_new (GPA_TYPE_KEY, GPA_NODE_ID (option));
+		key->option = GPA_NODE (option);
+		gpa_node_ref (key->option);
+		key->value = g_strdup (option->value);
+		child = (GPANode *) gpa_option_get_child_by_id (option, option->value);
+		if (child) {
+			GPANode *ich;
+			l = NULL;
+			for (ich = GPA_OPTION (child)->children; ich != NULL; ich = ich->next) {
+				GPANode *kch;
+				kch = gpa_option_create_key (GPA_OPTION (ich));
+				if (kch) l = g_slist_prepend (l, kch);
+			}
+			while (l) {
+				GPANode *kch;
+				kch = GPA_NODE (l->data);
+				l = g_slist_remove (l, kch);
+				kch->parent = GPA_NODE (key);
+				kch->next = key->children;
+				key->children = kch;
+			}
+			gpa_node_unref (child);
+			break;
+		} else {
+			g_warning ("Default was not in list (memory leak)");
+		}
+		break;
+	case GPA_OPTION_TYPE_ITEM:
+		/* No key for item */
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+	return (GPANode *) key;
+}
+
 GPANode *
 gpa_option_new_from_tree (xmlNodePtr tree)
 {
@@ -615,7 +693,7 @@ gpa_option_new_item_from_tree (xmlNodePtr tree, const guchar *id)
 
 	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
 	option->type = GPA_OPTION_TYPE_ITEM;
-	option->name = name;
+	option->name = gpa_node_attach (GPA_NODE (option), name);
 
 	while (l) {
 		GPANode *cho;
@@ -683,9 +761,25 @@ gpa_option_list_new_from_tree (xmlNodePtr tree)
 		options->children = option;
 	}
 
+	g_print ("GPAOptions: Created list %p\n", options);
+
 	return options;
 }
 
+/* Public interface */
+
+GPANode *
+gpa_option_create_key (GPAOption *option)
+{
+	g_return_val_if_fail (option != NULL, NULL);
+	g_return_val_if_fail (GPA_IS_OPTION (option), NULL);
+
+	if (GPA_OPTION_GET_CLASS (option)->create_key)
+		return GPA_OPTION_GET_CLASS (option)->create_key (option);
+
+	return NULL;
+}
+
 /* Strictly private helpers */
 
 GPAOption *
@@ -904,10 +998,8 @@ gpa_option_key_append_child (GPAOptionKey *option, GPAOption *child)
 static gboolean
 gpa_option_xml_check (xmlNodePtr node, gint def, gint val, gint name, gint children)
 {
-	xmlChar *xmlval;
-
 	if (GPA_UNAMBIGUOUS (def)) {
-		xmlval = xmlGetProp (node, "Default");
+		xmlChar *xmlval = xmlGetProp (node, "Default");
 		if (xmlval && GPA_FORBIDDEN (def)) {
 			g_warning ("Node does not have \"Default\" attribute");
 			xmlFree (xmlval);
@@ -917,10 +1009,11 @@ gpa_option_xml_check (xmlNodePtr node, gint def, gint val, gint name, gint child
 			xmlFree (xmlval);
 			return FALSE;
 		}
+		xmlFree (xmlval);
 	}
 
 	if (GPA_UNAMBIGUOUS (val)) {
-		xmlval = xmlGetProp (node, "Value");
+		xmlChar *xmlval = xmlGetProp (node, "Value");
 		if (xmlval && GPA_FORBIDDEN (val)) {
 			g_warning ("Node does not have \"Value\" attribute");
 			xmlFree (xmlval);
@@ -930,10 +1023,11 @@ gpa_option_xml_check (xmlNodePtr node, gint def, gint val, gint name, gint child
 			xmlFree (xmlval);
 			return FALSE;
 		}
+		xmlFree (xmlval);
 	}
 
 	if (GPA_UNAMBIGUOUS (name)) {
-		xmlval = gpa_xml_node_get_name (node);
+		xmlChar *xmlval = gpa_xml_node_get_name (node);
 		if (xmlval && GPA_FORBIDDEN (name)) {
 			g_warning ("Node does not have <Name> element");
 			xmlFree (xmlval);
@@ -943,6 +1037,7 @@ gpa_option_xml_check (xmlNodePtr node, gint def, gint val, gint name, gint child
 			xmlFree (xmlval);
 			return FALSE;
 		}
+		xmlFree (xmlval);
 	}
 
 	if (GPA_UNAMBIGUOUS (children)) {
