@@ -29,6 +29,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <ctype.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <popt.h>
@@ -298,11 +299,15 @@ gpa_node_describe (GPANode *node, GSList *nodes, gint level)
 	g_free (id);
 
 	if (!g_slist_find (nodes, node)) {
+		GPANode *prev;
 		nodes = g_slist_prepend (nodes, node);
+		prev = NULL;
 		for (child = gpa_node_get_child (node, NULL); child != NULL; child = gpa_node_get_child (node, child)) {
 			gpa_node_describe (child, nodes, level + 2);
-			gpa_node_unref (child);
+			if (prev) gpa_node_unref (prev);
+			prev = child;
 		}
+		if (prev) gpa_node_unref (prev);
 	}
 }
 
@@ -383,11 +388,27 @@ main (int argc, const char **argv)
 
 	if (gpa) {
 		GPANode *root;
+#if 0
+		GSList *nodes, *n;
+#endif
 		/* Describe GPA tree */
 		/* NB! GNOME_PRINT_CONFIG_NODE is not to be used in application now */
 		root = GNOME_PRINT_CONFIG_NODE (config);
 		gpa_node_describe (root, NULL, 0);
 		gnome_print_config_unref (config);
+#if 0
+		nodes = gpa_list_nodes ();
+		for (n = nodes; n != NULL; n = n->next) {
+			GPANode *node;
+			node = GPA_NODE (n->data);
+			if (!node->parent) {
+				g_print ("------------ STALE TREE -------------\n");
+				gpa_node_describe (node, NULL, 0);
+			} else {
+				g_print ("Node: %p id:%s qid:%d parent:%p\n", node, GPA_NODE_ID (node), node->qid, node->parent);
+			}
+		}
+#endif
 		exit (0);
 	}
 
@@ -483,6 +504,8 @@ main (int argc, const char **argv)
 		g_object_unref (G_OBJECT (pctx));
 	}
 
+	gnome_print_config_unref (config);
+
 	return 0;
 }
 
@@ -610,7 +633,6 @@ do_print_paragraph (GnomePrintContext *pc, GnomeFont *font, const guchar *b, con
 static void
 do_print_text (GnomePrintConfig *config, GnomePrintContext *ctx, GnomeFont *font, const guchar *buf, gint buflen)
 {
-	const GnomePrintUnit *unit;
 	gdouble pw, ph;
 	gdouble ml, mr, mt, mb;
 	gdouble x0, y0, x1, y1;
